@@ -4,6 +4,7 @@
 #include <re2/re2.h>
 #include "re2_xs.h"
 #include "compat-cophh.h"
+#include "compat-rx.h"
 
 #if PERL_VERSION > 10
 #define RegSV(p) SvANY(p)
@@ -116,7 +117,6 @@ RE2_comp(pTHX_
 #endif
         options.set_encoding(RE2::Options::EncodingLatin1);
 
-    // XXX: Probably should allow control of this somehow
     options.set_log_errors(false);
 
     SV *const max_mem = cophh_fetch_pvs(PL_curcop->cop_hints_hash,
@@ -125,6 +125,18 @@ RE2_comp(pTHX_
         options.set_max_mem(SvIV(max_mem));
     }
 
+    SV *const longest_match = cophh_fetch_pvs(PL_curcop->cop_hints_hash,
+            "re::engine::RE2::longest-match", 0);
+    if (SvOK(longest_match) && SvTRUE(longest_match)) {
+        options.set_longest_match(true);
+    }
+
+    SV *const never_nl = cophh_fetch_pvs(PL_curcop->cop_hints_hash,
+            "re::engine::RE2::never-nl", 0);
+    if (SvOK(never_nl) && SvTRUE(never_nl)) {
+        options.set_never_nl(true);
+    }
+
     // Try and compile first, if this fails we will fallback to Perl regex via
     // Perl_re_compile.
     const SV *const wrapped = stringify(aTHX_ flags, exp, plen);
@@ -167,23 +179,16 @@ RE2_comp(pTHX_
     rx->extflags = extflags;
     rx->engine   = &re2_engine;
 
-#if PERL_VERSION >= 11
+#if PERL_VERSION > 10
     rx->pre_prefix = SvCUR(wrapped) - plen - 1;
+#else
+    /* Preserve a copy of the original pattern */
+    rx->precomp = savepvn(exp, plen);
+    rx->prelen = (I32)plen;
 #endif
 
-#if PERL_VERSION == 10
-    rx->wraplen = SvCUR(wrapped);
-    rx->wrapped = savepvn(SvPVX(wrapped), SvCUR(wrapped));
-#else
     RX_WRAPPED(rx_sv) = savepvn(SvPVX(wrapped), SvCUR(wrapped));
     RX_WRAPLEN(rx_sv) = SvCUR(wrapped);
-#endif
-
-#if PERL_VERSION == 10
-    /* Preserve a copy of the original pattern */
-    rx->prelen = (I32)plen;
-    rx->precomp = savepvn(exp, plen);
-#endif
 
     /* Store our private object */
     rx->pprivate = (void *) ri;
@@ -224,6 +229,7 @@ RE2_exec(pTHX_ REGEXP * const rx, char *stringarg, char *strend,
     bool ok = ri->Match(
             re2::StringPiece(strbeg, strend - strbeg),
             stringarg - strbeg,
+            strend - strbeg,
             RE2::UNANCHORED,
             res, sizeof res / sizeof *res);
 
@@ -274,11 +280,18 @@ RE2_free(pTHX_ REGEXP * const rx)
     delete (RE2 *) RegSV(rx)->pprivate;
 }
 
+// Perl polluting our namespace, again.
+#undef Copy
 void *
 RE2_dupe(pTHX_ REGEXP * const rx, CLONE_PARAMS *param)
 {
 	PERL_UNUSED_ARG(param);
-    return RegSV(rx)->pprivate;
+
+    RE2 *previous = (RE2*) RegSV(rx)->pprivate;
+    RE2::Options options;
+    options.Copy(previous->options());
+
+    return new RE2 (re2::StringPiece(RX_WRAPPED(rx), RX_WRAPLEN(rx)), options);
 }
 
 SV *
